Package: fda
Version: 2.4.8
Date: 2018-06-29
Title: Functional Data Analysis
Author: J. O. Ramsay <ramsay@psych.mcgill.ca>, Hadley Wickham
        <h.wickham@gmail.com>, Spencer Graves
        <spencer.graves@prodsyse.com>, Giles Hooker <gjh27@cornell.edu>
Maintainer: J. O. Ramsay <ramsay@psych.mcgill.ca>
Depends: R (>= 2.10.0), splines, Matrix, graphics
Suggests: deSolve, R.matlab, quadprog, nlme, lattice, RCurl, zoo
Description: These functions were developed to support functional data
        analysis as described in Ramsay, J. O. and Silverman, B. W.
        (2005) Functional Data Analysis. New York: Springer.  They were
        ported from earlier versions in Matlab and S-PLUS.  An
        introduction appears in Ramsay, J. O., Hooker, Giles, and
        Graves, Spencer (2009) Functional Data Analysis with R and
        Matlab (Springer). The package includes data sets and script
        files working many examples including all but one of the 76
        figures in this latter book.  Matlab versions of the code and
        sample analyses are no longer distributed through CRAN, as they
        were when the book was published.  For those, ftp from
        <http://www.psych.mcgill.ca/misc/fda/downloads/FDAfuns/>
        There you find a set of .zip files containing the functions and
        sample analyses, as well as two .txt files giving instructions for
        installation and some additional information.
        The changes from Version 2.4.1 are fixes of bugs in density.fd and
        removal of functions create.polynomial.basis, polynompen, and 
        polynomial. These were deleted because the monomial basis
        does the same thing and because there were errors in the code.
License: GPL (>= 2)
URL: http://www.functionaldata.org
LazyData: true
Repository: CRAN
Repository/R-Forge/Project: fda
Repository/R-Forge/Revision: 771
Repository/R-Forge/DateTimeStamp: 2018-07-03 14:14:58
Date/Publication: 2018-07-05 13:19:38 UTC
NeedsCompilation: no
Packaged: 2018-07-03 14:30:11 UTC; rforge
Built: R 3.5.1; ; 2018-10-11 12:42:16 UTC; windows
