Matlab fda code is available from "www.functionaldata.org":
Click "Software", then "FDA Software" (at the bottom of the
page), then "Matlab".

Ramsay, Hooker and Graves (2009) Functional Data Analysis
with R and Matlab (Springer) said that the Matlab code could
be obtained from the Comprehensive R Archive Network (CRAN).
Since then, CRAN policies have changed, and we can no longer
distribute MATLAB code that way.  We are sorry for any
inconvenience.  Jim Ramsay, Giles Hooker, Spencer Graves