###
###
### Ramsey & Silverman (2006) Functional Data Analysis, 2nd ed. (Springer)
###
### DISCLAIMER:
###
### These script files are incomplete and where complete 
### do not always reproduce exactly the images in the books.  
### Reason for this include the following:
###
### (1) The authors were given permission to publish the results
###     of thier analyses but not always the data.
### (2) These script files were produced starting in 2007
###     not by the authors to the books but by volunteer(s)
###     (initially Spencer Graves).  The published analyses
###     were conducted over more than a decade, and the details
###     of exactly how the published analyses were performed were
###     not in some cases still available.   
### (3) The software and capabilities of computers available
###     have improved over time.  In some cases, this produces
###     a better fit today than at the times the published analyses
###     were prepared.  
###
### It is hoped that these script files may still be useful,
### even with their gaps and even when they do not reproduce exactly
### the published analyses.  
###
###
### ch. 3.  From functional data to smooth functions 
###
###
library(fda)
##
## Section 3.1.  Introduction 
##
# No figures in this section.  

##
## Section 3.2.  Some properties of functional data   
##
# pp. 41-42, Figure 3.1.  A cursive handwriting sample







##
## Section 3.3.  Representing functions by basis functions 
##

















##
## Section 3.4.  The Fourier basis system for periodic data 
##











##
## Section 3.5.  The spline basis system for open-ended data 
##










##
## Section 3.6.  Other useful basis systems 
##







##
## Section 3.7.  Choosing a scale for t 
##







##
## Section 3.8.  Further reading and notes  
##









